@@ -4,7 +4,8 @@ import discretize
 from ipywidgets import Dropdown, Text, FloatText, RadioButtons
 import matplotlib.pyplot as plt
 import numpy as np
-from geoh5py.objects import BlockModel, Curve, Octree
+import ezdxf
+from geoh5py.objects import BlockModel, Curve, Octree, Points, Surface
 from geoh5py.workspace import Workspace
 from ipywidgets.widgets import HBox, VBox
 from geoapps.plotting import plot_plan_data_selection
@@ -37,7 +38,7 @@ class Export(ObjectDataSelection):
     def __init__(self, **kwargs):
         kwargs = self.apply_defaults(**kwargs)
         self._file_type = Dropdown(
-            options=["ESRI shapefile", "csv", "geotiff", "UBC format"],
+            options=["ESRI shapefile", "csv", "geotiff", "UBC format", "dxf"],
             value="csv",
             description="Export type",
         )
@@ -60,10 +61,8 @@ class Export(ObjectDataSelection):
         super().__init__(**kwargs)
         self.trigger.description = "Export"
 
-        def save_selection(_):
-            self.save_selection()
 
-        self.trigger.on_click(save_selection)
+        self.trigger.on_click(self.save_selection)
 
         self._widget = VBox(
             [
@@ -72,7 +71,7 @@ class Export(ObjectDataSelection):
                 self.type_widget,
                 self.export_as,
                 self.trigger,
-                self.live_link_path,
+                self.export_directory,
             ]
         )
 
@@ -157,14 +156,14 @@ class Export(ObjectDataSelection):
         # Refresh the list of objects
         self.update_objects_list()
 
-        live_path = path.abspath(path.dirname(self.h5file))
-        if not path.exists(live_path):
-            mkdir(live_path)
+        export_path = path.abspath(path.dirname(self.h5file))
+        if not path.exists(export_path):
+            mkdir(export_path)
 
-        self.live_link_path._set_form_values(live_path, "")
-        self.live_link_path._apply_selection()
+        self.export_directory._set_form_values(export_path, "")
+        self.export_directory._apply_selection()
 
-    def save_selection(self):
+    def save_selection(self, _):
         if self.workspace.get_entity(self.objects.value):
             entity = self.workspace.get_entity(self.objects.value)[0]
         else:
@@ -186,7 +185,7 @@ class Export(ObjectDataSelection):
         if self.file_type.value == "csv":
             dataframe = object_2_dataframe(entity, fields=list(data_values.keys()))
             dataframe.to_csv(
-                f"{path.join(self.live_link_path.selected_path, self.export_as.value)}"
+                f"{path.join(self.export_directory.selected_path, self.export_as.value)}"
                 + ".csv",
                 index=False,
             )
@@ -206,28 +205,28 @@ class Export(ObjectDataSelection):
                         entity,
                         attribute=key,
                         file_name=path.join(
-                            self.live_link_path.selected_path, out_name
+                            self.export_directory.selected_path, out_name
                         ),
                         epsg=self.epsg_code.value,
                     )
                     print(
-                        f"Object saved to {path.join(self.live_link_path.selected_path, out_name) + '.shp'}"
+                        f"Object saved to {path.join(self.export_directory.selected_path, out_name) + '.shp'}"
                     )
             else:
                 out_name = re.sub("[^0-9a-zA-Z]+", "_", self.export_as.value)
                 export_curve_2_shapefile(
                     entity,
-                    file_name=path.join(self.live_link_path.selected_path, out_name),
+                    file_name=path.join(self.export_directory.selected_path, out_name),
                     epsg=self.epsg_code.value,
                 )
                 print(
-                    f"Object saved to {path.join(self.live_link_path.selected_path, out_name) + '.shp'}"
+                    f"Object saved to {path.join(self.export_directory.selected_path, out_name) + '.shp'}"
                 )
 
         elif self.file_type.value == "geotiff":
             for key in self.data.value:
                 name = (
-                    path.join(self.live_link_path.selected_path, self.export_as.value)
+                    path.join(self.export_directory.selected_path, self.export_as.value)
                     + "_"
                     + key
                     + ".tif"
@@ -249,7 +248,8 @@ class Export(ObjectDataSelection):
                         plt.colorbar(im, fraction=0.02)
                         plt.savefig(
                             path.join(
-                                self.live_link_path.selected_path, self.export_as.value
+                                self.export_directory.selected_path,
+                                self.export_as.value,
                             )
                             + "_"
                             + key
@@ -275,18 +275,46 @@ class Export(ObjectDataSelection):
                     data_obj = entity.get_data(key)[0]
                     models[
                         path.join(
-                            self.live_link_path.selected_path, self.export_as.value
+                            self.export_directory.selected_path, self.export_as.value
                         )
                         + "_"
                         + key
                         + ".mod"
                     ] = item[ind]
                 mesh.writeUBC(
-                    path.join(self.live_link_path.selected_path, self.export_as.value)
+                    path.join(self.export_directory.selected_path, self.export_as.value)
                     + ".msh",
                     models=models,
                 )
 
+            elif self.file_type.value == "dxf":
+                print(entity, "In DXF")
+                drawing = ezdxf.new(dxfversion='AC1024')
+                modelspace = drawing.modelspace()
+
+                if isinstance(entity, Curve):
+                    cell_parts = entity.parts[entity.cells[:, 0]]
+
+                    for part in np.unique(entity.parts).tolist():
+                        ind = cell_parts == part
+                        poly = modelspace.add_polyline3d(
+                            entity.vertices[entity.cells[ind, 0], :].tolist()
+                        )
+
+                elif isinstance(entity, Points):
+                    modelspace.add_point(entity.vertices.tolist())
+                elif isinstance(entity, Surface):
+                    surf = modelspace.add_mesh()
+                    surf.vertices = entity.vertices.to_list()
+                    surf.edges = entity.cells.tolist()
+                else:
+                    print("DXF export only supports Points, Curve and Surface objects")
+                    # poly.dxf.true_color = 0xFF0000
+
+                drawing.saveas(
+                    path.join(self.export_directory.selected_path, self.export_as.value)
+                    + ".dxf"
+                )
             else:
 
                 mesh = discretize.TensorMesh(
@@ -305,7 +333,7 @@ class Export(ObjectDataSelection):
                 ]
 
                 mesh.writeUBC(
-                    path.join(self.live_link_path.selected_path, self.export_as.value)
+                    path.join(self.export_directory.selected_path, self.export_as.value)
                     + ".msh"
                 )
 
@@ -322,7 +350,8 @@ class Export(ObjectDataSelection):
                             values = item
 
                         np.savetxt(
-                            path.join(self.live_link_path.selected_path, key) + ".mod",
+                            path.join(self.export_directory.selected_path, key)
+                            + ".mod",
                             values,
                         )
 
